import threading
import pynteractive.globals as pyn_globals
from pynteractive.datastruct import *
from pynteractive.graph import *
from pynteractive.tree import *
from pynteractive.map import *
from pynteractive.chart import *
from pynteractive.phylotree import *
from pynteractive.graphd3 import *
import atexit
import os
import pickle
from webserver import *


class WebServices:
	SERVERS_ON=False
	@staticmethod
	def webServerWorker():
		serveraddr = ('', pyn_globals.PORT)
		srvr = ThreadingServer(serveraddr, SimpleWS)
		srvr.serve_forever()

	@staticmethod
	def webSocketServerWorker():
		WSOCKserver = SimpleWebSocketServer("localhost", pyn_globals.WEBSOCKETPORT, JSCom)
		DataStruct._connect(WSOCKserver.sendAction)
		WSOCKserver.serveforever()

	@staticmethod
	def stopServers():
		a=open(os.path.join(os.path.expanduser("~"),".pynteractive"),'w')
		pickle.dump(Map.CACHE,a)
		a.close()

		if WebServices.SERVERS_ON:
			ThreadingServer.force_stop()
			WSOCKserver.close()

	@staticmethod
	def startServers():
		a=threading.Thread(target=WebServices.webServerWorker)
		b=threading.Thread(target=WebServices.webSocketServerWorker)
		a.setDaemon(True)
		b.setDaemon(True)
		a.start()
		b.start()

WebServices.startServers()
atexit.register(WebServices.stopServers)




HTMLCOLORS={'aliceblue':'#f0f8ff', 'antiquewhite':'#faebd7', 'aqua':'#00ffff', 'aquamarine':'#7fffd4', 'azure':'#f0ffff', 'beige':'#f5f5dc', 'bisque':'#ffe4c4', 'black':'#000000', 'blanchedalmond':'#ffebcd', 'blue':'#0000ff', 'blueviolet':'#8a2be2', 'brown':'#a52a2a', 'burlywood':'#deb887', 'cadetblue':'#5f9ea0', 'chartreuse':'#7fff00', 'chocolate':'#d2691e', 'coral':'#ff7f50', 'cornflowerblue':'#6495ed', 'cornsilk':'#fff8dc', 'crimson':'#dc143c', 'cyan':'#00ffff', 'darkblue':'#00008b', 'darkcyan':'#008b8b', 'darkgoldenrod':'#b8860b', 'darkgray':'#a9a9a9', 'darkgreen':'#006400', 'darkkhaki':'#bdb76b', 'darkmagenta':'#8b008b', 'darkolivegreen':'#556b2f', 'darkorange':'#ff8c00', 'darkorchid':'#9932cc', 'darkred':'#8b0000', 'darksalmon':'#e9967a', 'darkseagreen':'#8fbc8f', 'darkslateblue':'#483d8b', 'darkslategray':'#2f4f4f', 'darkturquoise':'#00ced1', 'darkviolet':'#9400d3', 'deeppink':'#ff1493', 'deepskyblue':'#00bfff', 'dimgray':'#696969', 'dodgerblue':'#1e90ff', 'firebrick':'#b22222', 'floralwhite':'#fffaf0', 'forestgreen':'#228b22', 'fuchsia':'#ff00ff', 'gainsboro':'#dcdcdc', 'ghostwhite':'#f8f8ff', 'gold':'#ffd700', 'goldenrod':'#daa520', 'gray':'#808080', 'green':'#008000', 'greenyellow':'#adff2f', 'honeydew':'#f0fff0', 'hotpink':'#ff69b4', 'indianred ':'#cd5c5c', 'indigo ':'#4b0082', 'ivory':'#fffff0', 'khaki':'#f0e68c', 'lavender':'#e6e6fa', 'lavenderblush':'#fff0f5', 'lawngreen':'#7cfc00', 'lemonchiffon':'#fffacd', 'lightblue':'#add8e6', 'lightcoral':'#f08080', 'lightcyan':'#e0ffff', 'lightgoldenrodyellow':'#fafad2', 'lightgray':'#d3d3d3', 'lightgreen':'#90ee90', 'lightpink':'#ffb6c1', 'lightsalmon':'#ffa07a', 'lightseagreen':'#20b2aa', 'lightskyblue':'#87cefa', 'lightslategray':'#778899', 'lightsteelblue':'#b0c4de', 'lightyellow':'#ffffe0', 'lime':'#00ff00', 'limegreen':'#32cd32', 'linen':'#faf0e6', 'magenta':'#ff00ff', 'maroon':'#800000', 'mediumaquamarine':'#66cdaa', 'mediumblue':'#0000cd', 'mediumorchid':'#ba55d3', 'mediumpurple':'#9370db', 'mediumseagreen':'#3cb371', 'mediumslateblue':'#7b68ee', 'mediumspringgreen':'#00fa9a', 'mediumturquoise':'#48d1cc', 'mediumvioletred':'#c71585', 'midnightblue':'#191970', 'mintcream':'#f5fffa', 'mistyrose':'#ffe4e1', 'moccasin':'#ffe4b5', 'navajowhite':'#ffdead', 'navy':'#000080', 'oldlace':'#fdf5e6', 'olive':'#808000', 'olivedrab':'#6b8e23', 'orange':'#ffa500', 'orangered':'#ff4500', 'orchid':'#da70d6', 'palegoldenrod':'#eee8aa', 'palegreen':'#98fb98', 'paleturquoise':'#afeeee', 'palevioletred':'#db7093', 'papayawhip':'#ffefd5', 'peachpuff':'#ffdab9', 'peru':'#cd853f', 'pink':'#ffc0cb', 'plum':'#dda0dd', 'powderblue':'#b0e0e6', 'purple':'#800080', 'rebeccapurple':'#663399', 'red':'#ff0000', 'rosybrown':'#bc8f8f', 'royalblue':'#4169e1', 'saddlebrown':'#8b4513', 'salmon':'#fa8072', 'sandybrown':'#f4a460', 'seagreen':'#2e8b57', 'seashell':'#fff5ee', 'sienna':'#a0522d', 'silver':'#c0c0c0', 'skyblue':'#87ceeb', 'slateblue':'#6a5acd', 'slategray':'#708090', 'snow':'#fffafa', 'springgreen':'#00ff7f', 'steelblue':'#4682b4', 'tan':'#d2b48c', 'teal':'#008080', 'thistle':'#d8bfd8', 'tomato':'#ff6347', 'turquoise':'#40e0d0', 'violet':'#ee82ee', 'wheat':'#f5deb3', 'white':'#ffffff', 'whitesmoke':'#f5f5f5', 'yellow':'#ffff00', 'yellowgreen':'#9acd32'}



import os
import re
import random

class Newick:
	releaf=re.compile("([^:]+)?(:[0-9.e-]+)?")

	class Node:
		RANDOMNAMES=set()
		def __init__(self,name=None,lgth=None):
			self.name=name
			self.length=lgth
			if self.length==None: self.length=0.001
			self.children=[]
			if not self.name:
				self.name=Newick.Node.randomName()
			self.dad=None

		def dadsToRoot(self):
			dads=[]
			x=self
			while x.dad:
				dads.append(x.dad)
				x=x.dad
			return dads[::-1]
	
		def append(self,x):
			self.children.append(x)

		def setParent(self,dad):
			self.dad=dad

		def __str__(self):
			cad=""
			if self.children:
				cad+="({0})".format(",".join([str(i) for i in self.children]))
			if self.name:
				cad+=self.name
			if self.length: cad+=":{0}".format(self.length)
			return cad

		@staticmethod
		def randomName():
			while True:
				name="clade-"+str(random.randint(1,99999))
				if name not in Newick.Node.RANDOMNAMES:
					Newick.Node.RANDOMNAMES.add(name)
					return name
			

	def __init__(self):
		self.root=None
		self.nodenames=set()
		self.internalNodes=set()
		self.leafs={}

	def readNewick(self,newick):
		self.nodenames=set()
		try:
			if os.path.isfile(newick):
				self.root,pos=self._parseTree(open(newick).read(),0)
			else:
				self.root,pos=self._parseTree(newick,0)
		except:
			raise Exception('Invalid format for Newick')

		self.internalNodes=self.getInternalNodes(self.root)

	def getInternalNodes(self,root):
		res=set()
		if not root.children: return res

		for i in root.children:
			res.update(self.getInternalNodes(i))

		res.add(root.name)
		
		return res
	
	def _parseTree(self,cad,pos):
		n=Newick.Node()
		assert cad[pos]=='('
		
		while True:
			if cad[pos]==')':
				if pos==len(cad)-1: break
				if cad[pos+1]==';': break
				if cad[pos+1] not in "),":
					name,lgth,pos=self._parseLeaf(cad,pos+1)
					if name: n.name=name
					n.length=lgth
					return n,pos
				else: break
			elif cad[pos]==';': 
				break
	
			if cad[pos+1]!='(': 
				name,lgth,pos=self._parseLeaf(cad,pos+1)
				nn=Newick.Node(name,lgth)
				self.leafs[name]=nn
				nn.setParent(n)
				n.append(nn)
			else:
				nn,pos=self._parseTree(cad,pos+1)
				n.append(nn)
				nn.setParent(n)
		return n,pos+1
	
	def _parseLeaf(self,cad,pos):
		length=None
		name=None
		i=pos
		while i<=len(cad) and cad[i] not in ",);":
			i+=1
		items=Newick.releaf.findall(cad[pos:i])
		if not items: raise Exception("Tree wrongly constructed")
		if items[0][0]: name=self.getName(items[0][0])
		if items[0][1]: length=float(items[0][1][1:])
	
		return name,length,i

	def getMonophyletics(self,nodes):
		assert type(nodes)==list
		found,clades=self._recur_getMonoPhyletics(nodes,self.root)
		clades.update([i for i in nodes if i in self.internalNodes])
		return clades

	def getCommonParent(self,nodes):
		assert type(nodes)==list
		parentslists=[]
		for i in nodes:
			parentslists.append(self.leafs[i].dadsToRoot())

		last=None
		for i in zip(*parentslists):
			if len(set(i))==1:
				last=i[0]
		return last


	def _recur_getMonoPhyletics(self,nodes,curNode):
		assert type(nodes)==list
		childres=[]
		res=set()
		found=False
		if curNode.children:
			for i in curNode.children:
				childres.append(self._recur_getMonoPhyletics(nodes,i))
			if all([i[0] for i in childres]):
				found=True
				res=set([curNode.name])
			else:
				res=reduce(set.union,[i[1] for i in childres])
		else:
			if curNode.name in nodes: return True,set([curNode.name])
			res=set()

		return found,res

	def getName(self,name):
		if not name in self.nodenames:
			self.nodenames.add(name)
			return name
		k=1
		newname=name+"#{0}".format(k)
		while newname in self.nodenames:
			k+=1
			newname=name+"#{0}".format(k)
		self.nodenames.add(newname)
		return newname

	def getNewick(self):
		return str(self.root)+";"
	__str__=getNewick




from pynteractive.visNetwork import *
import random

class Graph(VisNetwork):
	def __init__(self,name=None,directed=False):
		'''Creates a graph, It can be directed or not, if a name is not given it is created randomly'''
		VisNetwork.__init__(self,name,directed)

		

	def _refresh(self):
		'''DO NOT USE, is used for graphic representation, everytime a new window is opened'''
		for i in self.vertices.values():
	#			addNode: function (   id,      label,      title,      group,     shape,      color,      radius,       image, lng, lat){
			self._update("addNode",i["_id"],i["_label"],i["_title"],i["_group"],i["_shape"],i["_color"],i["_radius"],i["_image"])
		for i,j in self.edges.items():
			self._update("addEdge",i,j["_n1"],j["_n2"],j["_label"],j["_title"],j["_width"],j["_style"])

	def random(self,nn,ne):
		'''Creates a random Graph containing *nn* nodes and *ne* edges'''
		map(self.addNode,xrange(nn))
		for i in xrange(ne):
			self.addEdge(random.choice(xrange(nn)),random.choice(xrange(nn)))

	def fromCsv(self,fil,matrix=False,distances=False):
		'''Draws a network given a csv file
		
		The format for the CSV can be:

.. csv-table:: Pair of nodes connected
   :header: "n1", "n2"

   "a","b"
   "a","c"
   "c","d"

.. csv-table:: Node in first column connected to the ones in the next columns 
   :header: "n1", "n2"

   "a","b","d","e"
   "b","c","e"
   "e","f","a","d","e"

.. csv-table:: Network matrix (matrix=True,distances=False) 
   :header: " ","a", "b","c","d","e","f"

   "a",1,0,0,0,0,0
   "b",1,0,0,0,0,0	
   "c",0,1,1,1,0,0	
   "d",0,1,0,0,1,0	
   "e",1,0,0,0,1,1	
   "f",0,0,0,0,1,1	
		
.. csv-table:: Network matrix with distances (matrix=True,distances=True) 
   :header: " ","a", "b","c","d","e","f"

   "a",54,0,0,0,0,0
   "b",23,0,0,0,0,0	
   "c",0,10,2,1,0,0	
   "d",0,10,0,0,13,0	
   "e",56,0,0,0,1,1	
   "f",0,0,0,0,3,1	
   '''
		import pdb
		pdb.set_trace()
		csv=self.readCsv(fil)
		if not matrix:
			for i in csv:
				for j in i:
					if j not in self.vertices: self.addNode(j)
				for j in i[1:]:
					if not (i[0],j) in self.edges:
						self.addEdge(i[0],j)
		else:
			cols=csv[0][1:]
			rows=[i[0] for i in csv[1:]]
			if cols!=rows: raise Exception("Rows andColumns do not match")
			for i in rows:
				if i not in self.vertices: self.addNode(i)
			csv=[i[1:] for i in csv][1:]
			for i in range(len(rows)):
				for j in range(len(rows)):
					if csv[i][j]!='0':
						self.addEdge(rows[i],rows[j])




import inspect
import json
import mimetypes
import os
import sys
import tarfile
import threading
import urllib
import traceback
import base64
import StringIO
from pynteractive.SimpleWebSocketServer import WebSocket, SimpleWebSocketServer
import pynteractive.globals as pyn_globals
from pynteractive.datastruct import DataStruct
import re

Pynteractive_ws_MUTEX=threading.Lock()

if pyn_globals.VERSION<30:
	from BaseHTTPServer import HTTPServer
	from SimpleHTTPServer import SimpleHTTPRequestHandler
	from SocketServer import ThreadingMixIn
	from urlparse import urlparse,urlunparse 
	from urllib import unquote
else:
	from http.server import HTTPServer,SimpleHTTPRequestHandler
	from socketserver import ThreadingMixIn
	from urllib.parse import urlparse,urlunparse,unquote

class ThreadingServer(ThreadingMixIn, HTTPServer):
	stopped = False
	server=None

	def serve_forever(self):
		ThreadingServer.server=self
		while not self.stopped:
			self.handle_request()

	@staticmethod
	def force_stop():
		ThreadingServer.stopped = True
		urllib.urlopen("http://localhost:{0}/".format(pyn_globals.PORT)).read()

class FileMgr:
	def __init__(self,path=None,tarString=None):
		assert (path and not tarString) or (tarString and not path)
		if tarString:
			self.contents=self.iniTar(StringIO.StringIO(base64.b64decode(tarString)))
			self.METHOD="tar"
		else:
			self.iniPath('.')
			self.path='.'
			self.METHOD="path"
	
	def iniTar(self,tarStr):

		tar = tarfile.open(fileobj=tarStr,mode="r:gz")
		filelist=[]
			
		for tarinfo in tar:
			if tarinfo.isreg():
				filelist.append(tarinfo.name)
			elif tarinfo.isdir(): pass #print ("a directory.")
			else: pass #print ("something else.")
		return dict([(item,tar.extractfile(item).read()) for item in filelist])

	def iniPath(self,path):
		pass

	def __contains__ (self,item):
		if self.METHOD=='tar':
			return item in self.contents
		else:
			return os.path.isfile(os.path.join(self.path,item))
	
	def __getitem__(self,item):
		if self.METHOD=="tar":
			return self.contents[item]
		else:
			with open(os.path.join(self.path,item)) as q:
				res=q.read()
			return res

class JSCom(WebSocket):
	def __init__(self,*args,**kwargs):
		WebSocket.__init__(self,*args,**kwargs)
		funcs=[i for i in dir(self) if inspect.ismethod(getattr(self,i)) and getattr(self,i).__doc__!="FWCLASS"]
		self.dicFuncs={}
		self.dicArgs={}
		self.dataId=None
		for i in funcs:
			self.dicFuncs[i]=getattr(self,i)
			self.dicArgs[i]=inspect.getargspec(self.dicFuncs[i])

		self.MESSAGES={}

	def sendAction(self,fname,*params):
		data=json.dumps([fname]+list(params))
		with Pynteractive_ws_MUTEX:
			self.sendMessage(data)

	def handleMessage(self):
		orig=self.data
		try:
			dat=json.loads(str(orig))
			funcname=dat[0]
			args=dat[1]
			self.dicFuncs[funcname](**args)
		except:
			traceback.print_exc()
			print "Error processing","({0})".format(len(orig)),str(orig)[:30],"...",str(orig)[-30:]

	def handleConnected(self):
#		print self.address, 'connected'
		pass

	def handleClose(self):
		pass

	def attach(self,name):
		self.dataId=name
		self.server.attachConnToDataId(self,name)

	def graphDblClick(self,nodes):
		if nodes:
			DataStruct._OBJECTS[self.dataId]._doubleClick(nodes[0].encode())

	def treeNodeClick(self,node):
		DataStruct._OBJECTS[self.dataId].nodeClick(node.encode())

	def performAction(self,n,selectedNodes):
		selectedNodes=[str(i).encode() for i in selectedNodes]
		DataStruct._OBJECTS[self.dataId]._performAction(n,selectedNodes)

	def refresh(self,name):
		DataStruct._refreshData(name)

	def selectionSet(self,nodes):
		DataStruct._OBJECTS[self.dataId]._selectionSet(nodes)

	def downloadSVG(self,svg):
		q=open("/tmp/a.svg","w")
		q.write(svg)
		q.close()


class SimpleWS(SimpleHTTPRequestHandler):
	FILE_MGR=FileMgr(path='webfiles/') if not pyn_globals.WEBFILES else FileMgr(tarString=pyn_globals.WEBFILES)

	def do_GET(self):
		if self.path.startswith("/?"): resource="webfiles/index.html"
		else: resource='/'.join(["webfiles"]+self.path.split("?")[0].split("/"))
		resource=re.sub("/+","/",resource)
		if resource in SimpleWS.FILE_MGR:
			self.send_response(200)
			ext=self.path.split(".")[-1].lower()
			if ext in mimetypes.types_map:
				mime=mimetypes.types_map[ext]
				self.send_header("Content-Type", mime)
			else:
				self.send_header("Content-Type","text/html")

			self.end_headers()
			self.wfile.write(SimpleWS.FILE_MGR[resource])
		else:
			self.send_response(404)
			self.send_header("Content-Type", mimetypes.types_map[".html"])
			self.end_headers()
			self.wfile.write("<h1>File not found (404)</h1><br>We are so sorry :(<br><br><br>That's no so bad, we will die someday anyway<br>Have a good day :D".encode())

	def log_message(self, format, *args):
		return




from pynteractive.Network import *
import urllib
import json
import pickle
import os
import threading

class Map(Network):
	'''The Map class allows to plot data in an OpenStreet Map. The user can plot points of different sizes and colors
	as well as linking them with edges. In order to specify the location coordinates or places (eg. postcode) can be used.'''

	MAXTHREADS=15

	try: CACHE=pickle.load(open(os.path.join(os.path.expanduser("~"),".pynteractive")))
	except: CACHE={}

	RUNNINGTHREADS=threading.Semaphore(MAXTHREADS)
	THREADS=[]
	LOCK=threading.Lock()

	def __init__(self,name=None):
		'''Creates a map object'''
		Network.__init__(self,name,False)

	def _refresh(self):
		'''DO NOT USE, is used for graphic representation, everytime a new window is opened'''
		for i in self.vertices.values():
			self._update("addNode",i["_id"],i["_label"],'','','',i["_color"],i["_radius"],'',i["_lng"],i["_lat"])
		for i,j in self.edges.items():
			self._update("addEdge",i,j["_n1"],j["_n2"],'','','','','',j["_color"],j["_width"])

	def updateNode(self,node_id,radius=None,color=None,lng=None,lat=None,place=None,country=None):
		'''Updates a node in the Map'''
		Network.updateNode(self,node_id,radius=radius,color=color,lng=lng,lat=lat,place=place,country=country)
		i=self.vertices[node_id]
		self._update("updateNode",i["_id"],i["_label"],'','','',i["_color"],i["_radius"],'',i["_lng"],i["_lat"])

	def addNode(self,*args,**kwargs):
		th=threading.Thread(target=self._addNode,args=args,kwargs=kwargs)
		th.start()
		with Map.LOCK:
			Map.THREADS.append(th)


		with Map.LOCK:
			done=[]
			for i in Map.THREADS:
				if not i.is_alive(): done.append(i)
			for i in done: Map.THREADS.remove(i)


	def _addNode(self,node_id=None,radius=5,color='red',lng=None,lat=None,place=None,country=None):
		'''Adds a node to the map:

- node_id: identification of the node, if not specified it will be randomly generated
- radius: radius of the spot
- color: color name or html code (eg: red, #FF0000)
- lng,lat: you can provide longitude and latitude to place the spot in a geographical area
- place,country: If you don't use coordinates, you can specify a word the describes the place (eg, POSTCODE) and if you are worried that thare can be a comflict with another country you can use the country parameter that has to ISO 3166-1alpha2 compliant'''

		assert (lat==None and lng==None and place) or (not place and lat!=None and lng!=None)

		Map.RUNNINGTHREADS.acquire()
		try:
			if place:
				lng,lat=self._getLocation(place,country)
				if lng==None:
					print "Error getting coordinates for loading",place,country
					Map.RUNNINGTHREADS.release()
					return None,None
			
			_id,label=Network.addNode(self,node_id,node_id,radius=radius,color=color,lng=lng,lat=lat)
			self._update("addNode",node_id,node_id,'','','',color,radius,'',lng,lat)
		except: pass

		Map.RUNNINGTHREADS.release()
		return _id,label

	def addEdge(self,n1,n2,color='red',width=2):
		'''Ads and edge between two points given a color tag'''
		_id,label=Network.addEdge(self,n1,n2,'',color=color,width=width)
		self._update("addEdge",_id,n1,n2,'','','','','',color,width)
		return _id


	def focusNode(self,node_id):
		'''Zooms into a given node'''
		self._update("searchNode",node_id)

	def _getLocation(self,place,country=None):
		url="http://open.mapquestapi.com/nominatim/v1/search.php?format=json&limit=1&addressdetails=0&q={0}".format(place)
		if country: url+="&countrycodes={0}".format(country)
		if (place,country) in Map.CACHE: return Map.CACHE[(place,country)]
		try:
			a=urllib.urlopen(url)
			data=json.loads(a.read())[0]
			a.close()
			Map.CACHE[(place,country)]=(data['lon'],data['lat'])
			return data['lon'],data['lat']
		except:
			return None,None

	def delNode(self,node_id):
		'''Deletes a node from the map'''
		node,edges=Network.delNode(self,node_id)

		self._update("removeNode",node)
		for i in edges:
			self._update("removeEdge",i)

	def delEdge(self,eid):
		'''Deletes an edge given an egde ID'''
		edge=Network.delEdge(self,eid)
		self._update("removeEdge",eid)

	def clear(self):
		'''Deletes all the information plotted in the map'''
		e=self.getEdges()
		v=self.getNodes()
		for i in e:
			self.delEdge(i)
		for i in v:
			self.delNode(i)



from pynteractive.Network import *
import os
import re

class Tree(Network):
	def __init__(self,name=None,directed=None):
		self.nodenames=set()
		VisNetwork.__init__(self,name,True)
		self.root=None

	def getName(self,name):
		if not name in self.nodenames:
			self.nodenames.add(name)
			return name
		k=1
		newname=name+"#{0}".format(k)
		while newname in self.nodenames:
			k+=1
			newname=name+"#{0}".format(k)
		self.nodenames.add(newname)
		return newname




	def _refresh(self):
		if self.root:
			top=[self.root]
			self.addNode("_root")
			if self.root.children:
				topname,toplabel=self.addNode(self.root.name,label=' ',shape='dot',radius=0.01)
			else:
				topname,toplabel=self.addNode(self.root.name,shape='dot')
			self.root.name=topname
			self.addEdge("_root",topname)
			while top:
				rt=top.pop(0)
				for i in rt.children:
					top.append(i)
					if i.children:
						newnodename,newlabel=self.addNode(i.name,shape='dot',radius=0.01,label=' ')
					else:
						newnodename,newlabel=self.addNode(i.name,shape='dot')
					i.name=newnodename
					self.addEdge(rt.name,newnodename,length=50)

	def readNewick(self,newick):
		try:
			if os.path.isfile(newick):
				self.root,pos=self._parseTree(open(newick).read(),0)
			else:
				self.root,pos=self._parseTree(newick,0)
		except:
			raise Exception('Invalid format for Newick')

	def _parseTree(self,cad,pos):
		n=Tree.Node()
		assert cad[pos]=='('
		
		while True:
			if cad[pos]==')':
				if pos==len(cad)-1: break
				if cad[pos+1]==';': break
				if cad[pos+1] not in "),":
					name,lgth,pos=self._parseLeaf(cad,pos+1)
					n.name=name
					n.length=lgth
					return n,pos
				else: break
			elif cad[pos]==';': 
				break
	
			if cad[pos+1]!='(': 
				name,lgth,pos=self._parseLeaf(cad,pos+1)
				n.append(Tree.Node(name,lgth))
			else:
				nn,pos=self._parseTree(cad,pos+1)
				n.append(nn)
		return n,pos+1
	
	def _parseLeaf(self,cad,pos):
		length=None
		name=None
		i=pos
		while i<=len(cad) and cad[i] not in ",);":
			i+=1
		items=Tree.releaf.findall(cad[pos:i])
		if not items: raise Exception("Tree wrongly constructed")
		if items[0][0]: name=self.getName(items[0][0])
		if items[0][1]: length=float(items[0][1][1:])
	
		return name,length,i


if __name__=='__main__':
	print "1"
	_parseTree("(,,(,));",0)
	print "3"
	_parseTree("(A,B,(C,D));",0)
	print "4"
	_parseTree("(A,B,(C,D)E)F;",0)
	print "5"
	_parseTree("(:0.1,:0.2,(:0.3,:0.4):0.5);",0)
	print "6"
	_parseTree("(:0.1,:0.2,(:0.3,:0.4):0.5):0.0;",0)
	print "7"
	_parseTree("(A:0.1,B:0.2,(C:0.3,D:0.4):0.5);",0)
	print "8"
	_parseTree("(A:0.1,B:0.2,(C:0.3,D:0.4)E:0.5)F;",0)
	print "9"
	_parseTree("((B:0.2,(C:0.3,D:0.4)E:0.5)F:0.1)A;",0)




from datastruct import *
from itertools import izip

class Chart(DataStruct):
	TYPES={'bar':'Bar','scatter':'Scatter','line':'Line','stack':'Stack'}
	SHAPES=set(['circle', 'cross', 'triangle-up', 'triangle-down', 'diamond', 'square'])
	def __init__(self,name=None,type='line'):
		assert type in Chart.TYPES
		DataStruct.__init__(self,name)
		self.chartype=Chart.TYPES[type]
		self.data={}

	def _refresh(self):
		self._update("setChartType",self.chartype)
		for seriesName,values in self.data.items():
			dataset={"key":seriesName,"values":values}
			self._update("addChartData",dataset)

	def _addValue(self,series,x,y,**kwargs):
		assert not set(kwargs)-set(["size","shape"])
		if 'size' in kwargs:
			if type(kwargs['size'])==str: kwargs['size']=int(kwargs['size']) if kwargs['size'].isdigit() else float(kwargs['size'].replace(',','.'))
		if type(x)==str: x=int(x) if x.isdigit() else float(x.replace(',','.'))
		if type(y)==str: y=int(y) if y.isdigit() else float(y.replace(',','.'))

		data=dict({"x":x,"y":y}.items()+kwargs.items())
		self.data[series].append(data)
		return data

	def addSeries(self,seriesName,x=[],y=[],sizes=[],shapes=[],data=[],csv=None):
		'''Adds a series of data. A series of data has to have a name and a collection of (x,y) pairs that can be either integers or floats
		if the chart is a scatter you can add as well sizes and or shapes. 

		The available shapes are: 'circle', 'cross', 'triangle-up', 'triangle-down', 'diamond', 'square'

		There are several ways of adding data:

.. code:: python

		a=Chart()
		a.addSeries('dataset1',[1,2,3],[4,5,6])
		a.addSeries('dataset2',[1,2,3],[4,5,6],sizes=[.5,.5,.2],shapes=['circle','cross','square'])
		a.addSeries('dataset3',data=[[1,4],[2,5],[3,6]])
		a.addSeries('dataset4',data=[[1,4,.5,'circle'],[2,5,.5,'cross'],[3,6,.2,'square']])
		a.addSeries('dataset5',csv='file.csv') # The format has to the same as we pass it to data parameter
		'''
		assert seriesName not in self.data
		assert (not data and not csv and x) or (not x and not csv and data) or (not data and not x and csv)
		assert len(x)==len(y)
		assert not sizes or (sizes and len(sizes)==len(x)==len(shapes))
		assert len(set([len(i) for i in data]))<2

		self.data[seriesName]=[]

		if sizes:
			values=[]
			for _x,_y,_sh,_sz in izip(x,y,shapes,sizes):
				if _sh not in Chart.SHAPES:
					raise Exception('shape must be a valid id!')
				self._addValue(_x,_y,shape=_sh,size=_sz)
		elif x:
			for _x,_y in izip(x,y):
				self._addValue(seriesName,_x,_y)
		else:
			if csv:
				data=self.readCsv(csv)
			for i in data:
				if len(i)<2 or len(i)>4: raise Exception('You need at least 2 values in the data vector and not more than 4')
				if len(i)==3:
					sh='circle'
					sz=0.5
					if i[2] in Chart.SHAPES: sh=i[2]
					else: sz=i[2]
					self._addValue(seriesName,i[0],i[1],shape=sh,size=sz)
				elif len(i)==4:
					sh='circle'
					sz=0.5
					if i[2] in Chart.SHAPES: sh,sz=i[2],i[3]
					else: sh,sz=i[3],i[2]
					self._addValue(seriesName,i[0],i[1],shape=sh,size=sz)
				else:
					self._addValue(seriesName,i[0],i[1])

		dataset={"key":seriesName,"values":self.data[seriesName]}

		self._update("addChartData",dataset)


	def delSeries(self,seriesName):
		'''Deletes a series from the graph'''
		del self.data[seriesName]
		self._update("removeChartData",seriesName)

	def addValue(self,idSeries,x,y,**kwargs):
		'''Ads a value into a series, extra parameters can be size=N and shape='XXX' '''
		assert not set(kwargs)-set(["size","shape"])
		self._update("addSeriesData",idSeries,self._addValue(idSeries,x,y,**kwargs))

	def delValue(self,idSeries,x):
		'''Deletes a value from a series'''
		self._update("removeSeriesData",idSeries,x)
		self.data[idSeries]=[i for i in self.data[idSeries] if i['x']!=x]

	def changeType(self,type):
		'''Changes the chart type in the view, the following types are available: 'bar','scatter','line','stack' '''
		assert type in Chart.TYPES
		self.chartype=Chart.TYPES[type]
		self._update("setChartType",self.chartype)




from setuptools import setup, find_packages  # Always prefer setuptools over distutils
from codecs import open  # To use a consistent encoding
import os
import tarfile
import StringIO
import sys
import base64

def recurdir(d):
	e=[os.path.join(d,i)  for i in os.listdir(d)]
	ds=[i for i in e if os.path.isdir(i)]
	fs=[i for i in e if os.path.isfile(i)]

	for i in ds:
		fs+=recurdir(i)
	
	return fs


if "build" in sys.argv or "install" in sys.argv:
	webtarbuff=StringIO.StringIO()
	webtar=tarfile.open(fileobj=webtarbuff,mode="w:gz")
	for i in recurdir('webfiles'):
		webtar.add(i)
	
	webtar.close()
	webtarbuff.seek(0)
	webtarbuff=base64.b64encode(str(webtarbuff.read()))
	q=open("pynteractive/webfiles.py",'w')
	q.write("WEBFILES='''{0}'''".format(webtarbuff))
	q.close()


long_description='''Pynteractive is a library that uses different JS graphic suites (html)
in order to generate interactive graphics (Networks, trees, charts, maps...'''

setup(
	name='pynteractive',

	# Versions should comply with PEP440.  For a discussion on single-sourcing
	# the version across setup.py and the project code, see
	# https://packaging.python.org/en/latest/single_source_version.html
	version='1.1.1',

	description='Pynteractive, a suite to generate interactive Javascript graphics',
	long_description=long_description,

	# The project's main homepage.
	url='http://github.com/coelias/Pynteractive',

	# Author details
	author='Carlos del Ojo Elias, Oriol Mazariegos Canellas',
	author_email='deepbit@gmail.com, mazeitor@gmail.com',

	# Choose your license
	license='GPLv3',

	# See https://pypi.python.org/pypi?%3Aaction=list_classifiers
	classifiers=[
		# How mature is this project? Common values are
		#   3 - Alpha
		#   4 - Beta
		#   5 - Production/Stable
		'Development Status :: 3 - Alpha',

		# Indicate who your project is intended for
		'Intended Audience :: Science/Research',
		'Topic :: Scientific/Engineering :: Visualization',

		# Pick your license as you wish (should match "license" above)
		'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',

		# Specify the Python versions you support here. In particular, ensure
		# that you indicate whether you support Python 2, Python 3 or both.
		'Programming Language :: Python :: 2',
		'Programming Language :: Python :: 2.6',
		'Programming Language :: Python :: 2.7',
	],

	# What does your project relate to?
	keywords='plotting interactive graphics JS',

	# You can just specify the packages manually here if your project is
	# simple. Or you can use find_packages().
	packages=find_packages(),

	# List run-time dependencies here.  These will be installed by pip when your
	# project is installed. For an analysis of "install_requires" vs pip's
	# requirements files see:
	# https://packaging.python.org/en/latest/requirements.html
	install_requires=[],

	# List additional groups of dependencies here (e.g. development dependencies).
	# You can install these using the following syntax, for example:
	# $ pip install -e .[dev,test]
	extras_require = {
	},

	# If there are data files included in your packages that need to be
	# installed, specify them here.  If using Python 2.6 or less, then these
	# have to be included in MANIFEST.in as well.
	package_data={'pynteractive':["../"+i for i in recurdir("webfiles")] },

	# Although 'package_data' is the preferred approach, in some case you may
	# need to place data files outside of your packages.
	# see http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files
	# In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
	data_files=[],

	# To provide executable scripts, use entry points in preference to the
	# "scripts" keyword. Entry points provide cross-platform support and allow
	# pip to create the appropriate form of executable for the target platform.
	entry_points={ }
)

if "build" in sys.argv or "install" in sys.argv:
	q=open("pynteractive/webfiles.py",'w')
	q.write("WEBFILES=None".format(webtarbuff))
	q.close()



import random 
import webbrowser
import pynteractive.globals as pyn_globals
from collections import Counter
import traceback

class DataStruct:
	'''DataStruct is the basic data structure aimed to abstract any data set that will be represented in Pynteractive Web GUI. All
	different dataset inherit from DataStruct class, and datastruct class implements the basic mechanisms to communicate with the
	Pynteractive web server'''

	_JSConnector=None
	_OBJECTS={}

	def __init__(self,name=None):
		'''DataStruct is the superclass for all pynteractive objects. It requires a *name* although if it is not provided a random name will be assigned.'''
		if not name:
			name=self.__class__.__name__+"-"+"".join([random.choice("ABCDEF0123456789") for i in range(4)])
		ID=name
		if ID in DataStruct._OBJECTS:
			raise ("Object {0} already exists!".format(ID))
		DataStruct._OBJECTS[ID]=self
		self._ID=name
		self.actions={}
		self.actionid=1
		self.dclickfunc=None
		self.selectionHook=None
	
	def setDoubleClick(self,f):
		'''Sets a callback as a response to a double click event on a graph node'''
		self.dclickfunc=f

	def _doubleClick(self,node):
		if self.dclickfunc:
			self.dclickfunc(node)

	def selectionHandler(self,func):
		'''Sets a hook method that will be fired every time a selection is performed'''
		self.selectionHook=func

	def _selectionSet(self,nodes):
		if self.selectionHook:
			self.selectionHook(nodes)

	def _performAction(self,fid,params):
		fid=int(fid)
		if fid not in self.actions:
			print "Function not found!!!"
		else:	
			try:
				return self.actions[fid][1](params)
			except:
				print "Error calling method",fid
				print traceback.format_exc()


	def _update(self,func,*pars):
		'''This method calls a JS function via the WebSockets (USED INTERNALLY BY PYNTERACTIVE)'''
		if DataStruct._JSConnector:
			DataStruct._JSConnector(self._ID,func,*pars)

	def _refreshActions(self):
		for fid,(name,func) in self.actions.items():
			self._update('addAction',fid,name)

	@staticmethod
	def _refreshData(name):
		if name in DataStruct._OBJECTS:
			DataStruct._OBJECTS[name]._refreshActions()
			DataStruct._OBJECTS[name]._refresh()

	@staticmethod
	def _connect(updateFunc):
		DataStruct._JSConnector=updateFunc

	@staticmethod
	def readCsv(csvfile):
		'''readsCsv and returns the output as a vector of vectors. Detects delimiter automatically'''
		with open(csvfile) as f:
			csv=[i.strip() for i in f]
		if csv:
			delimiter=Counter([i for i in csv[0] if i in "\t ;,|"]).most_common(1)
			if delimiter:
				csv=[i.split(delimiter[0][0]) for i in csv]
		return csv

	def log(self,log):
		'''Dumps HTML code in the Log section in the web GUI'''
		self._update('addLog',log)

	def clearLog(self):
		'''Clears the contents of the log box in the web GUI'''
		self._update('clearLog')

	def closeView(self):
		'''Closes the browser window/tab containing the dataset GUI'''
		self._update("close")

	def view(self):
		'''This method opens the default web browser in the system showing the web visualization associated to the data'''
		webbrowser.open_new_tab("http://localhost:{0}/?dataid={1}&vtype={2}&wsport={3}".format(pyn_globals.PORT,self._ID,self.__class__.__name__,pyn_globals.WEBSOCKETPORT))

	def addAction(self,name,func):
		'''Actions can be added to each visualization

		An action consists of a **label** (*name*) shown in the web interface plus a **callback method** (*func*) called then the
		button associated to the label is clicked. The callback function must accept one parameter that depending on the visualization
		can be defferent (eg. list of nodes, sample id, etc...)
		
		example:

.. code:: python

		a=Graph()
		def myfunc(nodes):
		     print 'these are the selected nodes',nodes

		a.addAction('Print selected nodes',myfunc)
		a.view()

		# Now you can select some nodes in the GUI and click your action on the left side bar
		'''

		self.actions[self.actionid]=[name,func]
		self._update('addAction',self.actionid,name)
		self.actionid+=1



from pynteractive.datastruct import *
import os
from newickParser import *
from htmlcolors import HTMLCOLORS
import re

class PhyloTree(DataStruct):
	recolor=re.compile("^#[0-9a-f]{6}$")
	def __init__(self,name=None):
		DataStruct.__init__(self,name)
		self.newick=""
		self.features={}
		self.tipfeatures={}
		self.newick=Newick()
		self.tracks={}
		self.gradientTracks={}
		self.mcolorTracks={}
		self.ntracks=0
		self.trackBars={}
		self.nbars=0
		self.cladeMarks={}
		self.cladeColors={}

	def __getHexColor(self,trackn,val):
		_,color,minval,maxval,_=self.gradientTracks[trackn]
		if color in HTMLCOLORS: color=HTMLCOLORS[color]
		color=color.lower()
		if not PhyloTree.recolor.findall(color): raise Exception('Wrong color format (HTML valid color name or color code (#ff00ff))')
		rng=maxval-minval
		val-=minval
		val=rng-val
		val=float(val)/rng
		colvals=[int(color[i:i+2],16) for i in range(1,7,2)]
		colvals=[min([int(i+(255-i)*val),255]) for i in colvals]
		html="#"+"".join(["{0:0>2}".format(hex(i)[2:]) for i in colvals])
		return html


	def setData(self,newick=None):
		'''Draws the specified newick tree, It can be either a string containing the newick string or a path to a file'''
		self.newick.readNewick(newick)
		self._update('setData',str(self.newick))
		self.clearBars()
		self.clearTracks()
		self.tipfeatures={}

	def _refresh(self):
		self._update('setData',str(self.newick))
		for i,j in self.features.items():
			self._update('addPhyloFeat',i,*j)

		for i,j in self.tipfeatures.items():
			for k in j:
				self._update('addPhyloTipFeat',i,k)

		for i in range(self.ntracks):
			self._update('addTreeTrack')

		for trackn,data in self.tracks.items():
			for tipname,title in data[2].items():
				self._update('addTreeTrackFeature',trackn,tipname,data[1],title,False)

		for trackn,data in self.gradientTracks.items():
			for tipname,[title,value] in data[4].items():
				self._update('addTreeTrackFeature',trackn,tipname,self.__getHexColor(trackn,value),title,value)

		for trackn,data in self.mcolorTracks.items():
			for tipname,value in data[2].items():
				self._update('addTreeTrackFeature',trackn,tipname,data[1][value],value,value)

		for tipname,color in self.cladeMarks.items():
			self._update("markClade",tipname,color)

		for tipname,color in self.cladeColors.items():
			self._update("setCladeColor",tipname,color)

		self._refreshBars()

	def _refreshBars(self):
		for nbar,(description,color,minval,maxval,bars) in  self.trackBars.items():
			for tipname,value in bars.items():
				rng=maxval-minval
				val=value-minval
				val=rng-val
				val=1-float(val)/rng

				self._update('addTreeTrackBar',value,val,tipname,color,nbar,self.nbars)
			

	def getTips(self):
		'''Returns a set containing all the tip names found in the tree'''
		return [i.name for i in self.newick.leafs.values()]
			
	def nodeClick(self,nid):
		'''Method called when a node is clicked'''
		pass

	def selectTips(self,tips):
		'''Method usd to select nodes in the view (tree tips), you must provide tip labels'''
		self._update('selectNewickNodes',tips)


	def selectClade(self,tips):
		'''Select the clade containing a set of given tips'''
		dad=self.newick.getCommonParent(tips)
		self._update('selectNewickNodes',[dad.name])

	def clearSelection(self):
		'''Method to clear the current selection in the tree'''
		self._update('clearNewickSelection')

#	def addFeature(self,fid,shape,color,description=''):
#		'''Adds a feature available for the tree
#		
#- fid: Feature id (eg: INH-resist)
#- color: any html compatible color (eg: red,#f00)
#- shape: [circle,square,diamond,cross]
#- description: text to show in the legend'''
#		assert shape in ['circle','square','diamond','cross']
#		self.features[fid]=[shape,color,description]
#		self._update('addPhyloFeat',fid,shape,color,description)
#
#		
#	def addTipFeature(self,tid,fid):
#		'''Adds the feature `fid` to the tip `tid`'''
#		assert fid in self.features and tid in self.newick.nodenames
#		self.tipfeatures.setdefault(tid,[]).append(fid)
#		self._update('addPhyloTipFeat',tid,fid)
#
#	def delTipFeature(self,tid,fid):
#		'''Removes the feature `fid` to the tip `tid`'''
#		assert fid in self.tipfeatures[tid] and tid in self.newick.nodenames
#		self.tipfeatures[tid].remove(fid)
#		self._update('delPhyloTipFeat',tid,fid)

	def addTrack(self,description,color):
		'''Ads a track as an external ring. This is a BOOLEAN track, where any tip can be flagged in the track. When clicking a coloured tip on the track, all the tips flagged in the track will automatically be selected
		
- description: Text used for the legend
- color: color used for the flagged tips on the track

		Returns the track ID that you will have to use to add data to it'''

		self.ntracks+=1
		self.tracks[self.ntracks]=[description,color,{}]
		self._update('addTreeTrack')
		self._update('deleteBars')
		self._refreshBars()
		return self.ntracks

	def addMultiColorTrack(self,description,colordict):
		'''A Multicolor track is a track that can have random colors given a color dictionary eg: {"uk":'red',"france":'green',"germany":"#a2a2a2"}
- description: Text used for the legend
- colordict: Dictionary mapping values to colors
		
		Returns the track ID that you will have to use to add data to it '''

		self.ntracks+=1
		self.mcolorTracks[self.ntracks]=[description,colordict,{}]
		self._update('addTreeTrack')
		self._update('deleteBars')
		self._refreshBars()
		return self.ntracks

	def addGradientTrack(self,description,color,minval,maxval):
		'''A gradient Track is a track where tips in the tree can have a continuous value, and the color will be shown as a gradient color. You must specify minval and maxval of the track so that the gradient can be calculated
		
- description: Text used for the legend
- color: color used for the gradient on track
- minval: Minimum value of the range of values this track will have
- maxval: Minimum value of the range of values this track will have
		
		Returns the track ID that you will have to use to add data to it'''


		self.ntracks+=1
		self.gradientTracks[self.ntracks]=[description,color,minval,maxval,{}]
		self._update('addTreeTrack')
		self._update('deleteBars')
		self._refreshBars()
		return self.ntracks

	def addTrackFeature(self,trackn,tipname,value=False,title=None):
		'''Adds a data to a track:

- trackn: Track number to add data to
- tipname: The tip you want to add the feature to
- value: Only required/necessary if it's a gradient track
- title: Tooltip shown when hovering with the mouse 

.. code:: python

		p=PhyloTree()
		p.setData("file.newick")
		resistanteTrack=p.addTrack('Antibiotic resistante','red')
		coverageTrack=p.addTrack('Avg Depth coverage','blue',0,344)
		p.addTrackFeature(resistanceTrack,'sample1')
		p.addTrackFeature(resistanceTrack,'sample3')
		p.addTrackFeature(coverageTrack,'sampler1',25)
		p.addTrackFeature(coverageTrack,'sampler2',221)
		p.addTrackFeature(coverageTrack,'sampler3',344) '''

		assert tipname in self.newick.nodenames 
		if not 0<trackn<=self.ntracks: raise Exception("Track number does not exist")
		if trackn in self.gradientTracks: 
			if title==None: title=str(value)
			if type(value)==bool: raise Exception("You must specify a value for a gradient track")
			self.gradientTracks[trackn][4][tipname]=[title,value]
			self._update('addTreeTrackFeature',trackn,tipname,self.__getHexColor(trackn,value),title,value)
		elif trackn in self.tracks:
			if value!=False: raise Exception("This track does not accept values")
			self.tracks[trackn][2][tipname]=title
			self._update('addTreeTrackFeature',trackn,tipname,self.tracks[trackn][1],title,value)
		elif trackn in self.mcolorTracks:
			desc,colordict,feats=self.mcolorTracks[trackn]
			if value not in colordict: raise Exception("Value not found in color dictionary")
			feats[tipname]=value
			self._update('addTreeTrackFeature',trackn,tipname,colordict[value],value,value)
			

	def delTrackFeature(self,trackn,tipname):
		'''Deletes a feature present on a track'''

		assert trackn in self.tracks and tipname in self.newick.nodenames
		if tipname in self.tracks[trackn][2]: del self.tracks[trackn][2][tipname]
		self._update('delTreeTrackFeature',trackn,tipname)

	def addBar(self,description,color,minval,maxval):
		'''Outside the tracks, bars can be drawn showing extra information. You can have as many bars you want per sector (tip) (eg: if you want two bars in a barplot per tip, you call addBar twice and then you populate them with data
	
- description: Text description for the legend
- color: color used for the bar
- minval: minimum value of the data that the bars will show
- maxval: maximum value of the data that the bars will show

		Returns the Bar identifier to be used to add data to it 
		(See example below)
		'''

		self.nbars+=1
		self.trackBars[self.nbars]=[description,color,minval,maxval,{}]
		self._update('deleteBars')
		self._refreshBars()
		return self.nbars

	def addTrackBar(self,nbar,tipname,value):
		'''Adds data to a bar
	
- nbar: bar ID provided by addBar function
- tipname: name of the tip you will add data to
- value: Value the the bar will show 

.. code:: python

		p=PhyloTree()
		p.setData("file.newick")
		contaminationBars=p.addBar('contamination','green',0,28)
		p.addTrackBar(contaminationBars,'sample1',4)
		p.addTrackBar(contaminationBars,'sample2',22)
		p.addTrackBar(contaminationBars,'sample3',17) '''

		if not 0<nbar<=self.nbars: raise Exception("Bar number does not exist")
		_,color,minval,maxval,tipbars=self.trackBars[nbar]
		rng=maxval-minval
		val=value-minval
		val=rng-val
		val=1-float(val)/rng

		tipbars[tipname]=value

		self._update('addTreeTrackBar',value,val,tipname,color,nbar,self.nbars)

	def clearTracks(self):
		'''Clears all information in tracks'''
		self.tracks={}
		self.gradientTracks={}
		self.ntracks=0
		self._update("deleteTracks")

	def clearBars(self):
		'''Clears all bars around the tracks'''
		self.trackBars={}
		self.nbars=0
		self._update("deleteBars")

	def markClade(self,nodes,color):
		'''Marks the region of a clade given a fill color

- nodes: Is a list of tips/internalnodes
- color: HTML compatible color '''

		if type(nodes) not in [list,set]: nodes=[nodes]
		nodes=self.newick.getMonophyletics(nodes)

		for clade in nodes:
			self.cladeMarks[clade]=color
			self._update("markClade",clade,color)

	def unMarkClade(self,clade):
		'''Removes a mark for a clade'''
		del self.cladeMarks[clade]
		self._update("unMarkClade",clade)

	def clearCladeMarks(self):
		'''Removes all clade marks'''
		for i in self.cladeMarks.keys():
			self.unMarkClade(i)

	def setCladeColor(self,nodes,color):
		'''Sets tree color for a clade

- nodes: Is a list of tips/internalnodes
- color: HTML compatible color '''

		if type(nodes) not in [list,set]: nodes=[nodes]
		nodes=self.newick.getMonophyletics(nodes)

		for clade in nodes:
			self.cladeColors[clade]=color
			self._update("setCladeColor",clade,color)

	def clearCladeColor(self,clade):
		'''Clears tree color given a clade'''
		del self.cladeColors[clade]
		self._update("clearCladeColor",clade)



from pynteractive.datastruct import *

class Network(DataStruct):
	'''This class implements basic methods and datastructures to represent a Newtork. IT is used for either 
	Graphs or trees'''

	def __init__(self,name=None,directed=False):
		'''Creates a graph, It can be directed or not, if a name is not given it is created randomly'''
		DataStruct.__init__(self,name)

		self.vertices={}
		self.edges={}
		self.directed=directed

	def addNode(self,node_id=None,label=None,**kwargs):
		'''Ads a node into the network, you can specify a cwnode_id and a label, if no node_id is specified, it will be randomly generated (autoincremental id)
		and if label is None it will get the same value as the node_id

		Random parameters can be assigned to the node via \\*\\*kargs.

		It returns the tuple *node_id,label*.
		'''

		if node_id==None:
			i=1
			while True:
				if str(i) not in self.vertices:
					node_id=i
					break
				i+=1
		node_id=str(node_id)
		if not label: label=node_id
		else: label=str(label)

		data={"_id":node_id,"_label":label}
		data.update(dict([["_"+i,j] for i,j in kwargs.items()]))

		self.vertices[node_id]=data
		return node_id,label

	def updateNode(self,node_id,**kwargs):
		'''Updates parameters related to node *node_id* updating those parameters specified via \\*\\*kwargs'''
		assert node_id in self.vertices
		data=self.vertices[node_id]
		data.update(dict([["_"+i,j] for i,j in kwargs.items() if j]))


	def addEdge(self,n1,n2,label,**kwargs):
		'''Adds an edge from node n1 to  node n2, if it is not directed the order does not matter. The edge id will be a combination of n1,n2 and label (eg: addEdge('node1','node2','connection_node1_node2') --> edge_id = 'node1~node2~connection_node1_node2')'''
		n1,n2=str(n1),str(n2)
		assert n1 in self.vertices and n2 in self.vertices

		if not self.directed:
			if n1>n2:n1,n2=n2,n1

		_id="~".join([n1,n2,label])

		data={"_n1":n1,"_n2":n2,"_label":label}
		data.update(dict([["_"+i,j] for i,j in kwargs.items()]))
		self.edges[_id]=data
		return _id,label

	def delNode(self,node_id):
		'''Deletes a node from the network as well as the edges'''
		node_id=str(node_id)
		assert node_id in self.vertices 
		edges=[name for name,i in self.edges.items() if node_id in [i['_n1'],i['_n2']]]
		for i in edges:
			del self.edges[i]
		del self.vertices[node_id]
		return [node_id,edges]

	def delEdge(self,_id):
		'''Deletes a connection between two given nodes'''
		assert _id in self.edges
		del self.edges[_id]
		return _id

	def getEdges(self):
		'''Returns the edges of the Network --> ([edge_ids])'''
		return self.edges.keys()

	def getNodes(self):
		'''Returns the nodes of the Network --> ([node_ids])'''
		return self.vertices.keys()



from pynteractive.Network import *

class VisNetwork(Network):
	def __init__(self,name=None,directed=False):
		'''Implements a network for vis.js'''
		Network.__init__(self,name,directed)

	def addNode(self,node_id=None,label=None,title=None,group=None,shape='ellipse',color=None,radius=None,image=None):
		'''Adds a node to the network:

- node_id: identification of the node, if not specified it will be randomly generated
- label: Text that will be shown within the node
- title: Tooltip shown when hovering the node
- group: color group (numeric, eg: 1,2,3,4) used for clusterings
- shape: any of *ellipse*, *circle*, *box*, *database*, *image*, *circularImage*, *label*, *dot*, *star*, *triangle*, *triangleDown*, *square*
- color: color name or html code (eg: red, #FF0000)
- radius: radius of the shape
- image: Image in case the shape is image or circularImage'''

		assert not shape or shape in ['ellipse','circle','box','database','image','circularImage','label','dot','star','triangle','triangleDown','square']
		if shape in ['image','circularImage']: assert image!=None

		node_id,label=Network.addNode(self,node_id,label,title=title,group=group,shape=shape,color=color,radius=radius,image=image)

		self._update("addNode",node_id,label,title,group,shape,color,radius,image)
		return node_id,label

	def updateNode(self,node_id,label=None,title=None,group=None,shape=None,color=None,radius=None,image=None):
		'''Updates a node in the network'''
		Network.updateNode(self,node_id,label=label,title=title,group=group,shape=shape,color=color,radius=radius,image=image)
		i=self.vertices[node_id]
		self._update("updateNode",i["_id"],i["_label"],i["_title"],i["_group"],i["_shape"],i["_color"],i["_radius"],i["_image"])

	def addEdge(self,n1,n2,label=None,title=None,width=None,style=None,length=None):
		'''Adds an edge to a node, if it is not directed the order does not matter:

- label: label on the edge
- title: text shown when hovering the edge
- width: width in pixeld of the edge
- style: line style -> *line*, *arrow*, *arrow-center*, *dash-line*
- length: length of the edge'''

		assert not style or style in ['line','arrow','arrow-center','dash-line']
		if not self.directed: assert style not in ['arrow','arrow-center']

		if not label: label=''

		if self.directed:
			if not style: style='arrow'
		else:
			if not style: style='line'

		_id,label=Network.addEdge(self,n1,n2,label,title=title,width=width,style=style,length=length)
		self._update("addEdge",_id,n1,n2,label,title,width,style,length)
		return _id,label
	
	def delNode(self,node_id):
		'''Deletes a node from the network as well as the edges'''
		
		node,edges=Network.delNode(self,node_id)

		for i in edges:
			self._update("removeEdge",i)
		self._update("removeNode",node)

	def focusNode(self,id):
		'''Forces the GUI to zoom in into the specified node''' 
		self._update("searchNode",id)

	def delEdge(self,ed_id):
		'''Deletes a connection between two given nodes'''
		Network.delEdge(self,ed_id)
		self._update("removeEdge",ed_id)

	def clear(self):
		'''Clears the GUI by removing every node and edge in the network'''
		v=self.getEdges()
		n=self.getNodes()
		for i in v:
			self.delEdge(i)
		for i in n:
			self.delNode(i)
